/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <audacious/beepctrl.h>

#include "debug.h"

#include "../mediaplayer/mediaplayer.h"

#include "audacious.h"

#define MODULE_AUDACIOUS_MEDIAPLAYER_VERSION 1.0

// For CP1250->ISO8859-2 converter TODO: make it kadu-global
const char CODE_CP1250[]	= {0xb9, 0x9c, 0x9f, 0xa5, 0x8c, 0x8f};
const char CODE_ISO8859_2[] = {0xb1, 0xb6, 0xbc, 0xa1, 0xa6, 0xac};

AudaciousMediaPlayer *audacious;

// Kadu initializing functions
extern "C" int audacious_mediaplayer_init()
{
	audacious = new AudaciousMediaPlayer();
	bool res = mediaplayer->registerMediaPlayer(audacious, audacious);
	return res ? 0 : 1;
}

extern "C" void audacious_mediaplayer_close()
{
	mediaplayer->unregisterMediaPlayer();
	delete audacious;
	audacious = 0;
}

AudaciousMediaPlayer::AudaciousMediaPlayer()
{
	kdebugf();
}

AudaciousMediaPlayer::~AudaciousMediaPlayer()
{
	kdebugf();
}

// PlayerInfo

QString AudaciousMediaPlayer::getPlayerName()
{
	return "Audacious";
}

QString AudaciousMediaPlayer::getPlayerVersion()
{
	kdebugf();
	if (!xmms_remote_is_running(0))
		return "";

	return QString::number(xmms_remote_get_version(0));
}

QStringList AudaciousMediaPlayer::getPlayListTitles()
{
	kdebugf();
	QStringList list;

	// Firstly, how many items are in playlist?
	uint lgt = getPlayListLength();
	if (lgt == 0)
		return list;

	// Secondary, get all titles
	for ( uint cnt = 0; cnt < lgt; cnt++ )
		list.append(getTitle(cnt));

	return list;
}

QStringList AudaciousMediaPlayer::getPlayListFiles()
{
	kdebugf();
	QStringList list;

	// Firstly, how many items are in playlist?
	uint lgt = getPlayListLength();
	if (lgt == 0)
		return list;

	// Secondary, get all file names
	for ( uint cnt = 0; cnt < lgt; cnt++ )
		list.append(getFile(cnt));

	return list;
}

uint AudaciousMediaPlayer::getPlayListLength()
{
	kdebugf();
	return xmms_remote_get_playlist_length(0);
}

QString AudaciousMediaPlayer::getTitle(int position)
{
	kdebugf();
	if (!xmms_remote_is_running(0))
		return "";

	// Do we want current track title, or given track title?
	if (position == -1)
		position = xmms_remote_get_playlist_pos(0);

	QString title(convert(xmms_remote_get_playlist_title(0, position)));
	return title;
}

QString AudaciousMediaPlayer::getAlbum(int position)
{
	return QString::null;
}

QString AudaciousMediaPlayer::getArtist(int position)
{
	return QString::null;
}

QString AudaciousMediaPlayer::getFile(int position)
{
	kdebugf();

	// Do we want current track file name, or given track file name?
	if (position == -1)
		position = xmms_remote_get_playlist_pos(0);

	return convert(xmms_remote_get_playlist_file(0, position));
}

int AudaciousMediaPlayer::getLength(int position)
{
	kdebugf();

	// Do we want current track file name, or given track file name?
	if (position == -1)
		position = xmms_remote_get_playlist_pos(0);

	return xmms_remote_get_playlist_time(0, position);
}

int AudaciousMediaPlayer::getCurrentPos()
{
	kdebugf();
	return xmms_remote_get_output_time(0);
}

// PlayerCommands

void AudaciousMediaPlayer::nextTrack()
{
	kdebugf();
	xmms_remote_playlist_next(0);
}

void AudaciousMediaPlayer::prevTrack()
{
	kdebugf();
	xmms_remote_playlist_prev(0);
}

void AudaciousMediaPlayer::play()
{
	kdebugf();
	xmms_remote_play(0);
}

void AudaciousMediaPlayer::stop()
{
	kdebugf();
	xmms_remote_stop(0);
}

void AudaciousMediaPlayer::pause()
{
	kdebugf();
	xmms_remote_pause(0);
}

void AudaciousMediaPlayer::setVolume(int vol)
{
	kdebugf();
	xmms_remote_set_main_volume(0, vol);
}

void AudaciousMediaPlayer::incrVolume()
{
	kdebugf();
	int vol = xmms_remote_get_main_volume(0);
	vol += 2;
	if (vol > 100)
		vol = 100;

	setVolume(vol);
}

void AudaciousMediaPlayer::decrVolume()
{
	kdebugf();
	int vol = xmms_remote_get_main_volume(0);
	vol -= 2;
	if (vol < 0)
		vol = 0;

	setVolume(vol);
}

bool AudaciousMediaPlayer::isPlaying()
{
	kdebugf();
	return xmms_remote_is_playing(0);
}

bool AudaciousMediaPlayer::isActive()
{
	kdebugf();
	return xmms_remote_get_version(0) > 0;
}

QString AudaciousMediaPlayer::convert(QString str)
{
	for (unsigned int i = 0; i < sizeof(CODE_CP1250); i++)
	{
		str.replace(CODE_CP1250[i],CODE_ISO8859_2[i]);
	}
	return QString::fromLocal8Bit(str,-1);
}
